@@ -105,6 +105,17 @@ Nevow.Athena.PageWidget.methods(
         var d = self.remoteCalls[responseId];
         delete self.remoteCalls[responseId];
 
+        if (d === undefined) {
+            // If the transport is running then something strange has happened,
+            // otherwise just do nothing.
+            if (self.deliveryChannel.running) {
+                Divmod.msg(
+                    'Received response to unknown or already-responded call: ' +
+                        responseId);
+            }
+            return;
+        }
+
         if (success) {
             d.callback(result);
         } else {
@@ -492,7 +503,7 @@ Nevow.Athena.ReliableMessageDelivery.methods(
         for (var i = 0; i < self.requests.length; ++i) {
             self.requests[i].abort();
         }
-        self.requests = null;
+        self.requests = [];
         self.page.connectionLost('Connection closed by remote host');
     },
 
@@ -1405,9 +1416,7 @@ Nevow.Athena.IntrospectionWidget.methods(
  * Instantiate Athena Widgets.
  */
 Nevow.Athena.Widget._instantiateOneWidget = function(cls, node) {
-    Divmod.debug("widget", "Found Widget class " + cls + ", instantiating.");
     var inst = cls.get(node);
-    Divmod.debug("widget", "Widget class " + cls + " instantiated.");
     try {
         var widgetParent = Nevow.Athena.Widget.get(node.parentNode);
         widgetParent.addChildWidget(inst);
@@ -1417,7 +1426,6 @@ Nevow.Athena.Widget._instantiateOneWidget = function(cls, node) {
     }
     if (inst.loaded != undefined) {
         inst.loaded();
-        Divmod.debug("widget", "Widget class " + cls + " loaded.");
     }
     if (inst.nodeInserted != undefined) {
         inst.nodeInserted();
