@@ -21,12 +21,14 @@ from twisted.internet import defer
 from twisted.web import http
 from twisted.protocols.basic import LineReceiver
 from twisted.web.http_headers import Headers
+from twisted.web.test.requesthelper import DummyChannel
 
 from formless import iformless
 
 from nevow import inevow, context, athena, loaders, tags, appserver
 from nevow.jsutil import findJavascriptInterpreter, generateTestScript
 
+
 class FakeChannel:
     def __init__(self, site):
         self.site = site
@@ -389,7 +391,7 @@ def renderPage(res, topLevelContext=context.WebContext,
     ctx = topLevelContext(tag=res)
     ctx.remember(req, inevow.IRequest)
 
-    render = appserver.NevowRequest(None, True).gotPageContext
+    render = appserver.NevowRequest(DummyChannel(), True).gotPageContext
 
     result = render(ctx)
     result.addCallback(lambda x: req.accumulator)
